﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;

[ExecuteInEditMode]
public class GameManager : MonoBehaviour
{
    public List<Table> tables;
    public List<StaticNavObject> staticObjects;
    public NavMesh navMesh;
    public NavMeshSize navMeshSize;
    public PlayerController player;
    public List<int> pairsPerRound;
    public int pairsfor100;
    public int pairsfor80;
    public int pairsfor60;
    public int pairsfor40;
    public int pairsfor20;
    public List<NPC> availableFemaleNPCs;
    public List<NPC> availableMaleNPCs;
    public GameObject npcGroup;
    public Topics availableTopics;
    public List<NPC> npcs;

    public bool tablesWorking;
    public Text timer;
    public RectTransform resultPanel;
    public bool playerLeftRoom;
    private int currentRound;
    public bool showBubbles;
    public Text result;
    public Text detailedResult;

    public AudioSource backgroundMusic;
    public AudioSource backgroundNoise;
    public AudioSource chattingSound;

    public int[] stats;
    public int[] totalStats;

    public enum MatchingLevel
    {
        N80, N60, N40, N20, N0, P20, P40, P60, P80, P100
    }

    private static GameManager instance;

    public static GameManager Instance => instance == null ? GameObject.FindObjectOfType<GameManager>() : instance;

    private void Awake()
    {
        if (instance != null)
        {
            Destroy(gameObject);
        }
        else
        {
            instance = this;
        }
    }

    private void Start()
    {
        if (Application.isPlaying)
        {
            stats = new int[11];
            totalStats = new int[11];
            player.transform.position = new Vector3(-1000f, 0f, 0f);
            foreach (StaticNavObject staticNavObject in staticObjects)
            {
                foreach (NavMesh.Position blockPos in staticNavObject.blockedFields)
                {
                    navMesh.BlockNavMeshAt(new NavMesh.Position(staticNavObject.position.row + blockPos.row, staticNavObject.position.col + blockPos.col));
                }
            }
            foreach (Table t in tables)
            {
                foreach (NavMesh.Position blockPos in t.blockedFields)
                {
                    navMesh.BlockNavMeshAt(new NavMesh.Position(t.position.row + blockPos.row, t.position.col + blockPos.col));
                }
            }
            InitializeGame(0);
        }
    }

    public static void Shuffle<T>(IList<T> list)
    {
        int n = list.Count;
        while (n > 1)
        {
            n--;
            int k = Random.Range(0, n + 1);
            T value = list[k];
            list[k] = list[n];
            list[n] = value;
        }
    }

    private void AssignInterests(NPC npc)
    {
        List<int> usedTopics = new List<int>();
        for (int i = 0; i < NPC.likesAmount + NPC.dislikesAmount; ++i)
        {
            int topicToAdd = -1;
            while (topicToAdd == -1 || usedTopics.Contains(topicToAdd))
            {
                topicToAdd = Random.Range(0, availableTopics.list.Count);
            }
            npc.topics[i] = topicToAdd;
            usedTopics.Add(topicToAdd);
        }
    }

    private void MatchNPCs(NPC template, NPC newNPC, MatchingLevel level)
    {
        List<Topics.Topic> tempTopics = new List<Topics.Topic>(availableTopics.list);
        List<int> usedTopics = new List<int>();
        newNPC.topics = (int[])template.topics.Clone();
        switch (level)
        {
            case MatchingLevel.P100:
                // 5 matches
                break;
            case MatchingLevel.P80:
            case MatchingLevel.P60:
            case MatchingLevel.P40:
            case MatchingLevel.P20:
            case MatchingLevel.N0:
            case MatchingLevel.N20:
            case MatchingLevel.N40:
            case MatchingLevel.N60:
            case MatchingLevel.N80:
                // 4 matches, 1 different
                int topicToSwitch = Random.Range(0, 4);
                for (int i = 0; i < newNPC.topics.Length; ++i)
                {
                    if (i == topicToSwitch)
                    {
                        continue;
                    }
                    else
                    {
                        usedTopics.Add(template.topics[i]);
                    }
                }
                int newTopic = -1;
                while (newTopic == -1 || usedTopics.Contains(newTopic))
                {
                    newTopic = Random.Range(0, availableTopics.list.Count);
                }
                newNPC.topics[topicToSwitch] = newTopic;
                break;
        }
    }

    private void InitializeGame(int round)
    {
        currentRound = round;
        showBubbles = false;
        int pairs = pairsPerRound[Mathf.Min(round, pairsPerRound.Count - 1)];
        List<NPC> npcsF = new List<NPC>(pairs);
        List<NPC> npcsM = new List<NPC>(pairs);
        foreach (Table t in tables)
        {
            t.State = 0f;
        }
        List<int> usedWomen = new List<int>();
        List<int> usedMen = new List<int>();
        for (int i = 0; i < pairs; ++i)
        {
            int womanToCreate = -1;
            while (womanToCreate == -1 || usedWomen.Contains(womanToCreate))
            {
                womanToCreate = Random.Range(0, availableFemaleNPCs.Count);
            }
            usedWomen.Add(womanToCreate);
            NPC npcF = Instantiate<NPC>(availableFemaleNPCs[womanToCreate], new Vector3(0, -1000f, 0), Quaternion.identity, npcGroup.transform);
            AssignInterests(npcF);

            List<int> shuff = new List<int>(npcF.topics.Length);
            for (int k = 0; k < shuff.Capacity; ++k)
            {
                shuff.Add(k);
            }
            Shuffle<int>(shuff);
            npcF.topicPresentation = shuff;


            int manToCreate = -1;
            while (manToCreate == -1 || usedMen.Contains(manToCreate))
            {
                manToCreate = Random.Range(0, availableMaleNPCs.Count);
            }
            usedMen.Add(manToCreate);
            NPC npcM = Instantiate<NPC>(availableMaleNPCs[manToCreate], new Vector3(0, -1000f, 0), Quaternion.identity, npcGroup.transform);
            if (i < pairsfor100)
            {
                MatchNPCs(npcF, npcM, MatchingLevel.P100);
            }
            else if (i < pairsfor80)
            {
                MatchNPCs(npcF, npcM, MatchingLevel.P80);
            }
            else if (i < pairsfor60)
            {
                MatchNPCs(npcF, npcM, MatchingLevel.P60);
            }
            else if (i < pairsfor40)
            {
                MatchNPCs(npcF, npcM, MatchingLevel.P40);
            }
            else if (i < pairsfor20)
            {
                MatchNPCs(npcF, npcM, MatchingLevel.P20);
            }
            else
            {
                MatchNPCs(npcF, npcM, MatchingLevel.N0);
            }

            shuff = new List<int>(npcF.topics.Length);
            for (int k = 0; k < shuff.Capacity; ++k)
            {
                shuff.Add(k);
            }
            Shuffle<int>(shuff);
            npcM.topicPresentation = shuff;

            npcsF.Add(npcF);
            npcsM.Add(npcM);
        }
        Shuffle(npcsF);
        Shuffle(npcsM);
        player.allowedToMove = false;
        npcs.AddRange(npcsF);
        npcs.AddRange(npcsM);
        StartCoroutine(EntranceCoroutine(npcsF, npcsM));
    }

    private IEnumerator FadeVolume(AudioSource sound, float duration, float targetVolume)
    {
        float volumeStep = (targetVolume - sound.volume) / duration;
        while (sound.volume != targetVolume)
        {
            sound.volume += volumeStep * Time.deltaTime;
            yield return null;
        }
    }

    private IEnumerator EntranceCoroutine(List<NPC> npcsF, List<NPC> npcsM)
    {
        BlackFading.FadeOut();
        StartCoroutine(FadeVolume(backgroundMusic, 2f, 0.4f));
        StartCoroutine(FadeVolume(backgroundNoise, 2f, 0.1f));
        chattingSound.PlayDelayed(23.957f);
        yield return new WaitUntil(() => { return !BlackFading.IsFading; });
        player.currentPosition = navMesh.Entrance;
        player.transform.position = navMesh.Entrance.Coordinates;
        player.Turn(NavMesh.Direction.UP);
        player.NavigateTo(navMesh.PlayerStart);
        yield return new WaitWhile(() => { return player.IsNavigating; });
        player.Turn(NavMesh.Direction.DOWN);

        int tablesToSkip = tables.Count - npcsF.Count;
        int tableIndex = 0;
        NPC lastNPC = null;
        for (int i = 0; i < npcsF.Count; ++i)
        {
            while (Random.value < tablesToSkip / (1f * tables.Count))
            {
                ++tableIndex;
                --tablesToSkip;
            }
            Table table = tables[tableIndex];

            if (i != 0)
            {
                yield return new WaitForSeconds(0.8f);
                yield return new WaitUntil(() => { return !navMesh.NavMeshBlocked[navMesh.Entrance.row, navMesh.Entrance.col]; });
            }

            NPC npcF = npcsF[i];
            npcF.currentPosition = navMesh.Entrance;
            navMesh.BlockNavMeshAt(navMesh.Entrance);
            npcF.transform.position = navMesh.Entrance.Coordinates;
            npcF.Turn(NavMesh.Direction.UP);
            table.AssignNPC(npcF, true);

            yield return new WaitForSeconds(0.8f);
            yield return new WaitUntil(() => { return !navMesh.NavMeshBlocked[navMesh.Entrance.row, navMesh.Entrance.col]; });

            NPC npcM = npcsM[i];
            npcM.currentPosition = navMesh.Entrance;
            navMesh.BlockNavMeshAt(navMesh.Entrance);
            npcM.transform.position = navMesh.Entrance.Coordinates;
            npcM.Turn(NavMesh.Direction.UP);
            table.AssignNPC(npcM, false);

            lastNPC = npcM;
            ++tableIndex;
        }
        yield return new WaitWhile(() => { return lastNPC.IsNavigating; });
        StartCoroutine(GameCoroutine(npcsF.Count));
    }

    private IEnumerator ShowBubbles()
    {
        showBubbles = true;
        yield return new WaitForSeconds(2f);
        int currentTopic = 0;
        while (showBubbles)
        {
            foreach (NPC n in npcs)
            {
                if (Vector3.Distance(n.transform.position, player.transform.position) <= player.hearingRadius)
                {
                    int topicToShow = n.topicPresentation[currentTopic];
                    n.topicBubble.sprite = availableTopics.list[n.topics[topicToShow]].icon;
                    n.topicBubble.gameObject.SetActive(true);
                    if (topicToShow >= NPC.likesAmount)
                    {
                        n.dislikeIndicator.gameObject.SetActive(true);
                    }
                }
            }
            currentTopic = (currentTopic + 1) % (NPC.likesAmount + NPC.dislikesAmount);
            yield return new WaitForSeconds(1f);
            foreach (NPC n in npcs)
            {
                n.topicBubble.gameObject.SetActive(false);
                n.dislikeIndicator.gameObject.SetActive(false);
            }
            yield return new WaitForSeconds(2f);
        }
    }

    private IEnumerator GameCoroutine(int pairs)
    {
        player.allowedToMove = true;
        float timeToPlay = 60 * pairs;
        playerLeftRoom = false;
        tablesWorking = true;
        foreach (Table t in tables)
        {
            t.StartCoroutine(t.StateCoroutine());
        }
        StartCoroutine(ShowBubbles());
        yield return null;
        while (timeToPlay > 0f && !playerLeftRoom)
        {
            timeToPlay -= Time.deltaTime;
            int sec = ((int)timeToPlay) % 60;
            timer.text = ((int)timeToPlay) / 60 + ":" + (sec < 10 ? "0" : "") + sec;
            yield return null;
        }
        showBubbles = false;
        player.allowedToMove = false;
        tablesWorking = false;
        int score = 0;

        if (player.followingNPC != null)
        {
            foreach (NPC n in npcs)
            {
                if ((n.leftSide ? n.currentTable.rightNPC : n.currentTable.leftNPC) == null)
                {
                    score += (int)Table.CalculateMatchingLevel(player.followingNPC, n);
                    n.Leave();
                    stats[0] += 1;
                    totalStats[0] += 1;
                    break;
                }
            }
            player.followingNPC.Leave();
        }
        if (!playerLeftRoom)
        {
            player.NavigateTo(GameManager.Instance.navMesh.Entrance);
            yield return new WaitUntil(() => { return !player.IsNavigating; });
            player.transform.position = new Vector3(-1000f, 0, 0);
            navMesh.FreeNavMeshAt(navMesh.Entrance);
        }


        foreach (Table t in tables)
        {
            NPC left = t.leftNPC;
            NPC right = t.rightNPC;
            if (left != null || right != null)
            {
                yield return new WaitForSeconds(1f);
                if (left != null && right != null)
                {
                    score += (int)t.matchingLevel;
                    left.Leave();
                    t.RemoveNPC(true);
                    right.Leave();
                    t.RemoveNPC(false);
                    stats[((int)t.matchingLevel) + 1] += 2;
                    totalStats[((int)t.matchingLevel) + 1] += 2;
                }
                else if (left != null)
                {
                    left.Leave();
                    t.RemoveNPC(true);
                    stats[0] += 1;
                    totalStats[0] += 1;
                }
                else if (right != null)
                {
                    right.Leave();
                    t.RemoveNPC(false);
                    stats[0] += 1;
                    totalStats[0] += 1;
                }
            }
        }
        yield return new WaitUntil(() => { return npcs.Count == 0; });
        int percentage = -100 + (int)(score / (1.0 * pairs) * 20);
        result.text = "Score: " + percentage + "%";
        string details = "";
        bool firstLine = true;
        for (int i = 10; i >= 0; --i)
        {
            if (stats[i] != 0 || totalStats[i] != 0)
            {
                if (firstLine)
                {
                    firstLine = false;
                }
                else
                {
                    details += "\n";
                }
                details += (-100 + i * 20) + "% - x" + stats[i] + " (" + totalStats[i] + " total)";
            }
        }

        detailedResult.text = details;
        resultPanel.gameObject.SetActive(true);
        yield return new WaitUntil(() => { return Input.GetAxis("Submit") > 0f; } );
        BlackFading.FadeIn();
        yield return new WaitUntil(() => { return !BlackFading.IsFading; });
        resultPanel.gameObject.SetActive(false);

        CleanupGame();
    }

    private void CleanupGame()
    {
        foreach (Table t in tables)
        {
            t.RemoveNPC(true);
            t.State = 0;
            t.lightSource.gameObject.SetActive(false);
            t.RemoveNPC(false);
        }
        for (int i = 0; i < npcGroup.transform.childCount; ++i)
        {
            Transform child = npcGroup.transform.GetChild(0);
            navMesh.FreeNavMeshAt(child.GetComponent<NPC>().currentPosition);
            Destroy(child.gameObject);
        }
        InitializeGame(currentRound + 1);
    }
}
