﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Table : StaticNavObject
{
    public enum SeatState
    {
        FREE, RESERVED, OCCUPIED
    }

    public SpriteRenderer tablefoot;
    public SpriteRenderer candleFlame;
    public SpriteRenderer candleBody;
    public SpriteMask mask;
    public Light lightSource;
    public Sprite candleLit;
    public Sprite candleOut;
    public AudioSource candleAudio;
    private Animator anim;

    private SeatState leftSeatState = SeatState.FREE;
    private SeatState rightSeatState = SeatState.FREE;

    public NPC leftNPC;
    public NPC rightNPC;

    public NavMesh.Position leftSeatPosition => position.left;
    public NavMesh.Position rightSeatPosition => position.right;

    public GameManager.MatchingLevel matchingLevel;
    public float decayspeed = 0.2f;

    private float state;
    public float State {
        get {
            return state;
        }
        set {
            mask.alphaCutoff = 1 - state;
            Vector3 newPos = candleFlame.transform.localPosition;
            newPos.y = Mathf.Lerp(0f, 1.36f, state);
            candleFlame.transform.localPosition = newPos;

            if (anim != null)
            {
                Debug.Log("anim == null");
                anim.SetBool("candleDead", state <= 0f);
            }
            state = value;
        }
    }
    private NPC lastRemovedNPC;

    private IEnumerator AnimateCandle()
    {
        while (GameManager.Instance.tablesWorking)
        {
            lightSource.spotAngle = Mathf.Max(0f, 100 * state - 3 * Random.value);
            yield return null;
        }
    }

    protected override void Awake()
    {
        base.Awake();
        int layer = 1000 - 10 * position.row;
        tablefoot.sortingOrder = layer;
        candleBody.sortingOrder = layer + 1;
        candleFlame.sortingOrder = layer + 2;
        State = 0;
        anim = GetComponentInChildren<Animator>();
    }

    public bool AssignNPC(NPC n, bool leftSide)
    {
        if (leftSide && leftSeatState == SeatState.FREE)
        {
            leftSeatState = SeatState.RESERVED;
            n.NavigateTo(leftSeatPosition);
            n.currentTable = this;
            n.currentState = NPC.State.MOVINGTOTABLE;
            n.leftSide = leftSide;
            return true;
        }
        else if (!leftSide && rightSeatState == SeatState.FREE)
        {
            rightSeatState = SeatState.RESERVED;
            n.NavigateTo(rightSeatPosition);
            n.currentTable = this;
            n.currentState = NPC.State.MOVINGTOTABLE;
            n.leftSide = leftSide;
            return true;
        }
        return false;
    }

    public void RemoveNPC(bool leftSide)
    {
        NPC n = leftSide ? leftNPC : rightNPC;
        if (leftSide)
        {
            leftNPC = null;
            leftSeatState = SeatState.FREE;
        }
        else
        {
            rightNPC = null;
            rightSeatState = SeatState.FREE;
        }
        if ((leftSide ? rightNPC : leftNPC) != null)
        {
            lastRemovedNPC = n;
        }
        else
        {
            lastRemovedNPC = null;
        }
    }

    public void Arrive(NPC n, bool leftSide)
    {
        if (leftSide)
        {
            leftSeatState = SeatState.OCCUPIED;
            leftNPC = n;
        }
        else
        {
            rightSeatState = SeatState.OCCUPIED;
            rightNPC = n;
        }
        n.currentState = NPC.State.TALKING;
        if (n != lastRemovedNPC)
        {
            State = 1f;
        }
        if (leftNPC != null && rightNPC != null)
        {
            matchingLevel = CalculateMatchingLevel(leftNPC, rightNPC);
            decayspeed = (int)matchingLevel < 7 ? 1f / (30 + ((int)matchingLevel) * 15) : 0f; 
        }
    }

    public static GameManager.MatchingLevel CalculateMatchingLevel(NPC left, NPC right)
    {
        int currentLevel = 4;
        for (int i = 0; i < NPC.likesAmount; ++i)
        {
            bool found = false;
            for (int j = 0; j < NPC.likesAmount; ++j)
            {
                if (left.topics[i] == right.topics[j])
                {
                    ++currentLevel;
                    found = true;
                    break;
                }
            }
            if (!found)
            {
                for (int j = NPC.likesAmount; j < NPC.likesAmount + NPC.dislikesAmount; ++j)
                {
                    if (left.topics[i] == right.topics[j])
                    {
                        --currentLevel;
                        break;
                    }
                }
            }
        }
        for (int i = NPC.likesAmount; i < NPC.likesAmount + NPC.dislikesAmount; ++i)
        {
            bool found = false;
            for (int j = 0; j < NPC.likesAmount; ++j)
            {
                if (left.topics[i] == right.topics[j])
                {
                    --currentLevel;
                    found = true;
                    break;
                }
            }
            if (!found)
            {
                for (int j = NPC.likesAmount; j < NPC.likesAmount + NPC.dislikesAmount; ++j)
                {
                    if (left.topics[i] == right.topics[j])
                    {
                        ++currentLevel;
                        break;
                    }
                }
            }
        }
        return (GameManager.MatchingLevel)currentLevel;
    }

    public IEnumerator StateCoroutine()
    {
        StartCoroutine(AnimateCandle());
        while (GameManager.Instance.tablesWorking)
        {
            if (leftNPC != null && rightNPC != null)
            {
                lightSource.gameObject.SetActive(true);
                State = Mathf.Max(state - decayspeed * Time.deltaTime, 0f);
                if (state == 0f)
                {
                    candleAudio.Play();
                    //candleFlame.sprite = candleOut;
                    leftNPC.Leave();
                    rightNPC.Leave();
                    GameManager.Instance.stats[0] += 2;
                    GameManager.Instance.totalStats[0] += 2;
                }
            }
            yield return null;
        }
    }
}
