﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;

public class PlayerController : Person
{
    public Text text;
    public Text indicator;

    public float hearingRadius;
    public Light hearingRadiusIndicator;

    public NPC followingNPC;
    public bool allowedToMove;

    private void Awake()
    {
        isMoving = false;
        lookingDirection = NavMesh.Direction.DOWN;
        indicator.text = "v";

        int layer = 1000 - 10 * currentPosition.row;
        personSprite.sortingOrder = layer;
        allowedToMove = false;
    }

    private void OnValidate()
    {
        transform.position = NavMesh.GetPositionCoordinates(currentPosition);
        float angle = 2 * Mathf.Atan(hearingRadius / Mathf.Abs(hearingRadiusIndicator.transform.localPosition.z));
        hearingRadiusIndicator.spotAngle = angle * Mathf.Rad2Deg;
    }

    private void Start()
    {
        StartCoroutine(Controls());
    }

    public override void Turn(NavMesh.Direction dir)
    {
        base.Turn(dir);
        switch (dir)
        {
            case NavMesh.Direction.UP:
                indicator.text = "^";
                break;
            case NavMesh.Direction.RIGHT:
                indicator.text = ">";
                break;
            case NavMesh.Direction.DOWN:
                indicator.text = "v";
                break;
            case NavMesh.Direction.LEFT:
                indicator.text = "<";
                break;
        }
    }

    public override void MoveTo(NavMesh.Position newPos)
    {
        NavMesh.Position current = currentPosition;
        base.MoveTo(newPos);
        if (followingNPC != null)
        {
            followingNPC.MoveTo(current);
        }
    }

    private IEnumerator Controls()
    {
        float lastSubmitValue = Input.GetAxis("Submit");
        while (true)
        {
            if (Input.GetAxis("Cancel") > 0f)
            {
                allowedToMove = false;
                AsyncOperation loading = SceneManager.LoadSceneAsync("MainMenu");
                loading.allowSceneActivation = false;
                BlackFading.FadeIn();
                yield return new WaitUntil(() => { return !BlackFading.IsFading; });
                loading.allowSceneActivation = true;
            }
            text.text = Input.GetAxis("Horizontal") + " " + Input.GetAxis("Vertical");
            NavMesh navMesh = GameManager.Instance.navMesh;
            if (allowedToMove && !isMoving)
            {
                bool wantToMove = true;
                if (Input.GetAxis("Horizontal") > 0)
                {
                    Turn(NavMesh.Direction.RIGHT);
                }
                else if (Input.GetAxis("Horizontal") < 0)
                {
                    Turn(NavMesh.Direction.LEFT);
                }
                else if (Input.GetAxis("Vertical") > 0)
                {
                    Turn(NavMesh.Direction.UP);
                }
                else if (Input.GetAxis("Vertical") < 0)
                {
                    Turn(NavMesh.Direction.DOWN);
                }
                else
                {
                    wantToMove = false;
                    if (Input.GetAxis("Submit") > 0f && lastSubmitValue == 0f)
                    {
                        if (currentPosition.Equals(GameManager.Instance.navMesh.Entrance) && lookingDirection == NavMesh.Direction.DOWN)
                        {
                            GameManager.Instance.playerLeftRoom = true;
                            transform.position = new Vector3(-1000, 0, 0);
                            GameManager.Instance.navMesh.FreeNavMeshAt(currentPosition);
                        }
                        else
                        {
                            NavMesh.Position lookedAt = currentPosition.GetNeighbor(lookingDirection);
                            if (lookedAt != null)
                            {
                                foreach (Table t in GameManager.Instance.tables)
                                {
                                    if (followingNPC != null && t.position.Equals(lookedAt))
                                    {
                                        if (t.leftNPC == null && !t.leftSeatPosition.Equals(currentPosition))
                                        {
                                            t.AssignNPC(followingNPC, true);
                                            this.followingNPC = null;
                                        }
                                        else if (t.rightNPC == null && !t.rightSeatPosition.Equals(currentPosition))
                                        {
                                            t.AssignNPC(followingNPC, false);
                                            this.followingNPC = null;
                                        }
                                    }
                                    else if (lookedAt.Equals(t.leftSeatPosition) && t.leftNPC != null)
                                    {
                                        NPC oldFollower = followingNPC;
                                        NPC newFollower = t.leftNPC;

                                        newFollower.currentTable = null;
                                        newFollower.currentState = NPC.State.FOLLOWING;

                                        if (followingNPC != null)
                                        {
                                            newFollower.currentPosition = oldFollower.currentPosition;
                                            oldFollower.currentPosition = t.leftSeatPosition;
                                            oldFollower.currentTable = t;
                                            newFollower.transform.position = oldFollower.transform.position;
                                            oldFollower.transform.position = t.leftSeatPosition.Coordinates;

                                            t.Arrive(oldFollower, true);
                                        }
                                        else
                                        {
                                            t.RemoveNPC(true);
                                        }
                                        followingNPC = newFollower;
                                    }
                                    else if (lookedAt.Equals(t.rightSeatPosition) && t.rightNPC != null)
                                    {
                                        NPC oldFollower = followingNPC;
                                        NPC newFollower = t.rightNPC;

                                        newFollower.currentTable = null;
                                        newFollower.currentState = NPC.State.FOLLOWING;

                                        if (followingNPC != null)
                                        {
                                            newFollower.currentPosition = oldFollower.currentPosition;
                                            oldFollower.currentPosition = t.rightSeatPosition;
                                            oldFollower.currentTable = t;
                                            newFollower.transform.position = oldFollower.transform.position;
                                            oldFollower.transform.position = t.rightSeatPosition.Coordinates;

                                            t.Arrive(oldFollower, false);
                                        }
                                        else
                                        {
                                            t.RemoveNPC(false);
                                        }
                                        followingNPC = newFollower;
                                    }
                                }
                            }
                        }
                    }
                }
                if (wantToMove && navMesh.GetWalkable(currentPosition, lookingDirection))
                {
                    NavMesh.Position newPos = navMesh.GetNewPosition(currentPosition, lookingDirection);
                    MoveTo(newPos);
                }
            }
            lastSubmitValue = Input.GetAxis("Submit");
            yield return null;
        }
    }
}
